from dotenv import load_dotenv, find_dotenv

dotenv_path = find_dotenv()
load_dotenv(".env", override=True)

from .log import Log
import logging
Log_ = Log(console_level = logging.WARNING, # 显示控制台的等级
             log_file_name="app.log")
logger = Log_.logger
Log_.set_super_log(logger.critical) # 控制superlog 打印的等级 默认是最高级单独存储一个文件
super_log = Log_.super_log # 调试工具
inference_save_case = False




from pro_craft import AsyncIntel
from digital_life.models import *
import pandas as pd


class Test_Evals:
    def __init__(self,inference_save_case = False,model_name = "doubao-1-5-pro-256k-250115"):
        self.inters = AsyncIntel(model_name = model_name)
        self.inference_save_case = inference_save_case



async def test_memory_card1():
    inters = AsyncIntel(model_name = "doubao-1-5-pro-32k-250115")
    await inters._evals(prompt_id="memorycard-generate-content",OutputFormat = Document,ExtraFormats_list = [Chapter])

    inters.df.to_csv("tests/memory_card.csv",index=False)
    inters.draw_data()

    

"""


    async def evals(self,save_path = None):
        await self.inters._evals(prompt_id="memorycard-score",OutputFormat = MemoryCardScore) # 单一型
        await self.inters._evals(prompt_id="memorycard-merge",OutputFormat = MemoryCard2)
        await self.inters._evals(prompt_id="memorycard-polish",OutputFormat = MemoryCard)
        
        await self.inters._evals(prompt_id="memorycard-format",OutputFormat = MemoryCardGenerate2)
   
        if save_path:
            self.inters.df.to_csv(save_path,index=False)
        
        self.inters.draw_data()
 


    async def evals(self,save_path = None):
        df = pd.DataFrame({"name":[],'status':[],"score":[],"total":[],"bad_case":[]})
        await self.inters._evals(prompt_id="user-overview",OutputFormat = ContentVer, df = df)
        await self.inters._evals(prompt_id="user-relationship-extraction",OutputFormat = CharactersData, df = df)
        await self.inters._evals(prompt_id="avatar-brief",OutputFormat = BriefResponse, df = df)
        # await self.inters._evals(prompt_id="avatar-personality-extraction",OutputFormat = "", df = df)
        await self.inters._evals(prompt_id="avatar-desensitization",OutputFormat = BriefResponse, df = df)



        
        if save_path:
            df.to_csv(save_path,index=False)
        
        self.draw_data(df=df)

    async def evals(self,save_path = None):
        df = pd.DataFrame({"name":[],'status':[],"score":[],"total":[],"bad_case":[]})
        await self.inters._evals(prompt_id="biograph_material_init",OutputFormat = "", df = df)
        await self.inters._evals(prompt_id="biograph_material_add",OutputFormat = "", df = df)
        await self.inters._evals(prompt_id="biograph-outline",OutputFormat = "what", df = df)
        await self.inters._evals(prompt_id="biograph-paid-title",OutputFormat = BiographPaidTitle, df = df)
        await self.inters._evals(prompt_id="biograph-brief",OutputFormat = "", df = df)
        await self.inters._evals(prompt_id="biograph-extract-person-name",OutputFormat = "", df = df)
        await self.inters._evals(prompt_id="biograph-extract-place",OutputFormat = "", df = df)
        await self.inters._evals(prompt_id="biograph-extract-material",OutputFormat = "", df = df)
        await self.inters._evals(prompt_id="biograph-writer",OutputFormat = "", df = df)
        await self.inters._evals(prompt_id="biograph-free-writer",OutputFormat = Biography_Free, df = df)
"""


import logging
import os
from logging.handlers import RotatingFileHandler, TimedRotatingFileHandler

class Log:
    _instance = None
    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, console_level = logging.INFO, log_file_name="app.log"):
        self.Console_LOG_LEVEL = console_level
        self.log_file_name = log_file_name
        self.LOG_FILE_PATH = os.path.join("logs", log_file_name)
        self.logger = self.get_logger()
        self.super_log_level = self.logger.critical

    def get_logger(self):
        logger = logging.getLogger()
        # logger.setLevel(self.LOG_LEVEL)
        if not logger.handlers:
            # --- 4. 配置 Formatter (格式化器) ---
            formatter = logging.Formatter(
                "%(asctime)s - %(levelname)s - %(name)s - %(funcName)s - %(module)s:%(lineno)d - %(message)s"
            )
            # --- 5. 配置 Handler (处理器) ---

            # 5.1 控制台处理器 (StreamHandler)
            console_handler = logging.StreamHandler()
            console_handler.setLevel(self.Console_LOG_LEVEL)  # 控制台只显示 INFO 及以上级别的日志
            console_handler.setFormatter(formatter)
            logger.addHandler(console_handler)


            # 文件

            # 5.2 信息数据

            # RotatingFileHandler: 按文件大小轮转
            # maxBytes: 单个日志文件的最大字节数 (例如 10MB)
            # backupCount: 保留的旧日志文件数量
            file_handler = RotatingFileHandler(
                self.LOG_FILE_PATH,
                maxBytes=10 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler.setLevel(logging.INFO)  # 文件中显示所有指定级别的日志
            file_handler.setFormatter(formatter)
            logger.addHandler(file_handler)

            # 5.3 错误警告日志

            file_handler_debug = RotatingFileHandler(
                self.LOG_FILE_PATH.replace('.log','_err.log'),
                maxBytes=10 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler_debug.setLevel(logging.WARNING)  # 文件中显示所有指定级别的日志
            file_handler_debug.setFormatter(formatter)

            file_handler_cri = RotatingFileHandler(
                self.LOG_FILE_PATH.replace('.log','_slog.log'),
                maxBytes=5 * 1024 * 1024,  # 10 MB
                backupCount=5,
                encoding="utf-8",
            )
            file_handler_cri.setLevel(logging.CRITICAL)  # 文件中显示所有指定级别的日志
            file_handler_cri.setFormatter(formatter)
            logger.addHandler(file_handler_cri)
        return logger
    
    def set_super_log(self,logger_info):
        self.super_log_level = logger_info

    def super_log(self,s, target: str = "target",logger = None):
        logger = logger or self.super_log_level
        logger("\n" + "=="*25 + target +"=="*25 +   f"\n   type:  {str(type(s))}" + f"\ncontent:  {s}")





# 工具箱
import re
from digital_life.models import MemoryCards

def memoryCards2str(memory_cards: MemoryCards):
    memoryCards_str = ""
    memoryCards_time_str = ""
    for memory_card in memory_cards:
        memory_card_str = memory_card["title"] + "\n" + memory_card["content"] + "\n"
        memoryCards_str += memory_card_str
        memoryCards_time_str += "\n"
        memoryCards_time_str += memory_card["time"]
    return memoryCards_str, memoryCards_time_str


def extract_last_user_input(dialogue_text):
    """
    从多轮对话文本中提取最后一个 user 的输入内容。

    Args:
        dialogue_text: 包含多轮对话的字符串。

    Returns:
        最后一个 user 的输入内容字符串，如果未找到则返回 None。
    """
    pattern = r"(?s).*user:\s*(.*?)(?=user:|$)"

    match = re.search(pattern, dialogue_text)

    if match:
        # group(1) 捕获的是最后一个 user: 到下一个 user: 或字符串末尾的内容
        return match.group(1).strip()
    else:
        return None
    

def extract_json(text: str) -> str:
    """从文本中提取python代码
    Args:
        text (str): 输入的文本。
    Returns:
        str: 提取出的python文本
    """
    pattern = r"```json([\s\S]*?)```"
    matches = re.findall(pattern, text)
    if matches:
        return matches[0].strip()  # 添加strip()去除首尾空白符
    else:
        return ""  # 返回空字符串或抛出异常，此处返回空字符串


def extract_article(text: str) -> str:
    """从文本中提取python代码
    Args:
        text (str): 输入的文本。
    Returns:
        str: 提取出的python文本
    """
    pattern = r"```article([\s\S]*?)```"
    matches = re.findall(pattern, text)
    if matches:
        return matches[0].strip()  # 添加strip()去除首尾空白符
    else:
        return ""  # 返回空字符串或抛出异常，此处返回空字符串



#
import asyncio
from pro_craft import AsyncIntel
from digital_life.models import PersonInfo, CharactersData, ContentVer, BriefResponse
from digital_life.utils import memoryCards2str, extract_article
from digital_life import inference_save_case
from digital_life import super_log
import pandas as pd


###

class DigitalAvatar:
    def __init__(self,inference_save_case = False,model_name = "doubao-1-5-pro-256k-250115"):
        self.inters = AsyncIntel(model_name = model_name)
        self.inference_save_case = inference_save_case


    async def desensitization(self, memory_cards: list[str]) -> list[str]:
        """
        数字分身脱敏 0100
        0100
        """
        tasks = []
        for memory_card in memory_cards:
            tasks.append(self.inters.intellect_format(
                                input_data=memory_card.get("content"),
                                prompt_id="avatar-desensitization",
                                version = None,
                                inference_save_case=self.inference_save_case,
                                OutputFormat = ContentVer,
                                 ))

        results = await asyncio.gather(*tasks, return_exceptions=False)

        for i, memory_card in enumerate(memory_cards):
            memory_card["content"] = results[i].get("content")
        return memory_cards
    
    async def personality_extraction(self, memory_cards: list[dict],action:str,old_character:str) -> str:
        """
        数字分身性格提取 0099
        """
        memoryCards_str, _ = memoryCards2str(memory_cards)

        output_format = """"""
        result = await self.inters.intellect(
                                    input_data="\n操作方案:\n" + action + "\n旧人物性格:\n" + old_character +"\n记忆卡片:\n" +  memoryCards_str,
                                    output_format=output_format,
                                    prompt_id ="avatar-personality-extraction",
                                    version = None,
                                    inference_save_case = self.inference_save_case,
                                    )
        
        
        super_log(result,"数字分身personality_extraction-result")
        
        result_content = extract_article(result)
        return result_content

    async def abrief(self, memory_cards: list[dict]) -> dict:
        """
        数字分身介绍 0098
        """
        
        memoryCards_str, _ = memoryCards2str(memory_cards)
        result = await self.inters.intellect_format(
                                input_data="聊天历史:\n" + memoryCards_str,
                                prompt_id="avatar-brief",
                                version = None,
                                inference_save_case=self.inference_save_case,
                                OutputFormat = BriefResponse,
                                 )
        return result

    async def auser_relationship_extraction(self,text: str) -> dict:
        """
        用户关系提取 0097
        # TODO
        """
        result = await self.inters.intellect_format(
                        input_data="聊天历史" + text,
                        prompt_id="user-relationship-extraction",
                        version = None,
                        inference_save_case=self.inference_save_case,
                        OutputFormat = CharactersData,
                        ExtraFormats=[PersonInfo]
                            )
        return result

    async def auser_overview(self,action: str,old_overview: str, memory_cards: list[dict],
                             version = None,
                             ) -> str:
        """
        用户概述 0096
        """
        memoryCards_str, _ = memoryCards2str(memory_cards)
        result = await self.inters.intellect_format(
                input_data="\n操作方案:\n" + action + "\n旧概述文本:\n" + old_overview +"\n记忆卡片:\n" +  memoryCards_str,
                prompt_id="user-overview",
                version = None,
                inference_save_case=self.inference_save_case,
                OutputFormat = ContentVer,
                    )
        return result.get("content")







# server
# 推荐算法
from fastapi import APIRouter, Depends, HTTPException, status
from digital_life.models import UpdateItem, DeleteResponse, DeleteRequest, QueryItem
from digital_life.core.recommended import Recommend
from digital_life import logger

router = APIRouter(tags=["recommended"])

rep = Recommend()

@router.post(
    "/update",  # 推荐使用POST请求进行数据更新
    summary="更新或添加文本嵌入",
    description="将给定的文本内容与一个ID关联并更新到Embedding池中。",
    response_description="表示操作是否成功。",
)
def recommended_update(item: UpdateItem):
    try:
        if item.type in [0, 1, 2]:  # 上传的是卡片
            rep.update(text=item.text, id=item.id, type=item.type)
        else:
            logger.error(f"Error updating EmbeddingPool for ID '{item.id}': {e}")
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=f"Failed to update embedding for ID '{item.id}': {e}",
            )

        return {"status": "success", "message": f"ID '{item.id}' updated successfully."}

    except ValueError as e:  # 假设EmbeddingPool.update可能抛出ValueError
        logger.warning(f"Validation error during update: {e}")
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail=str(e))
    except Exception as e:
        logger.error(f"Error update for ID '{item.id}': {e}")
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed update {e}",
        )


@router.post("/delete", response_model=DeleteResponse, description="delete")
async def delete_server(request: DeleteRequest):
    try:
        rep.delete(id=request.id)  # 包裹的内核函数
        ########
        return DeleteResponse(
            status="success",
        )
    except ValueError as e:  # 假设EmbeddingPool.update可能抛出ValueError
        logger.warning(f"Validation error during ")
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail=str(e))
    except Exception as e:
        logger.error(f"Error delete")
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed to delete ",
        )

@router.post(
    "/search_biographies_and_cards",
    summary="搜索传记和记忆卡片",
    description="搜索传记和记忆卡片",
    response_description="搜索结果列表。",
)
async def recommended_biographies_and_cards(query_item: QueryItem):
    try:
        clear_result = await rep.recommended_biographies_and_cards(user_id = query_item.user_id,
                                                                   timestamp=query_item.timestamp,
                                                                   current=query_item.current,
                                                                    size=query_item.size,
             )

        return {
            "status": "success",
            "result": clear_result,
            "query": query_item.user_id,
        }

    except Exception as e:
        logger.error(
            f"Error search_biographies_and_cards '{query_item.user_id}': {e}"
        )
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed to search_biographies_and_cards: {e}",
        )

@router.post(
    "/search_figure_person",
    description="搜索数字分身的",
)
async def recommended_figure_person(query_item: QueryItem):
    try:

        clear_result = await rep.recommended_figure_person(user_id = query_item.user_id,
                                                           timestamp = query_item.timestamp,
                                                           current=query_item.current,
                                                           size=query_item.size,
             )
        return {
            "status": "success",
            "result": clear_result,
            "query": query_item.user_id,
        }

    except Exception as e:
        logger.error(
            f"Error search_figure_person '{query_item.user_id}': {e}"
        )
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Failed to search_figure_person: {e}",
        )





from fastapi import APIRouter
from digital_life.core import DigitalAvatar
from digital_life.models import BriefResponse, MemoryCards, AvatarXGRequests
from fastapi import FastAPI, HTTPException, status
router = APIRouter(tags=["digital_avatar"])
da = DigitalAvatar(inference_save_case = False,
                   model_name = "doubao-1-5-pro-32k-250115")

@router.post(
    "/brief", response_model=BriefResponse, description="数字分身介绍"
)
async def brief_server(request: MemoryCards):
    try:
        memory_cards = request.model_dump()["memory_cards"]
        result = await da.abrief(memory_cards=memory_cards)
        return BriefResponse(
        title=result.get("title"),
        content=result.get("content"),
        tags=result.get("tags")[:2],)
    except Exception as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"DigitalAvatar.abrief Error : {e}",
        )


@router.post("/personality_extraction")
async def digital_avatar_personality_extraction(request: AvatarXGRequests):
    """数字分身性格提取"""
    try:
        memory_cards = request.model_dump()["memory_cards"]
        result = await da.personality_extraction(memory_cards=memory_cards,action = request.action,old_character = request.old_character)
        return {"text": result}
    except Exception as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"DigitalAvatar.personality_extraction Error : {e}",
        )


@router.post("/desensitization")
async def digital_avatar_desensitization(request: MemoryCards):
    """
    数字分身脱敏
    """
    try:
        memory_cards = request.model_dump()["memory_cards"]
        result = await da.desensitization(memory_cards=memory_cards)
        memory_cards = {"memory_cards": result}
        return MemoryCards(**memory_cards)
    except Exception as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"DigitalAvatar.desensitization Error : {e}",
        )
